import sys
from copy import deepcopy

import numpy as np
import pandas as pd

from wj_analysis.facebook import polarity_distribution

ERR_SYS = "\nSystem error: "


class TopPolarityScoreFB:
    def top_post(self, df_sentiment, df_posts, df_pages, threshold=10):
        """
        This method returns the top posts of the day calculating the relative polarity of the comments.

        Parameters
        ----------
        df_posts : DataFrame
            DataFrame with the posts of Facebook
        df_pages : DataFrame
            DataFrame with the pages of Facebook

        Returns
        df : DataFrame
            add column 'rel_polarity' that calculates the relative polarity of the comments, organized from the most meaningful to the least meaningful
        ----------
        """

        METHOD_NAME = "posts"

        PAGES_COLUMNS = ["page_id", "name"]
        POSTS_COLUMNS = ["page_id", "post_id", "message", "permalink_url"]
        SENTIMENT_COLUMNS = ["post_id", "sentiment"]

        OUTPUT_COLUMNS = [
            "page_id",
            "post_id",
            "message",
            "permalink_url",
            "name",
            "sentiment",
            "count_comments",
            "rel_polarity",
        ]

        if len(df_sentiment) > 0 and len(df_posts) > 0:
            try:
                df_pages_temp = deepcopy(df_pages[PAGES_COLUMNS])
                df_getpolarity = deepcopy(df_posts[POSTS_COLUMNS])
                df_getpolarity["name"] = df_getpolarity["page_id"].map(
                    dict(df_pages_temp[PAGES_COLUMNS].values)
                )
                df_getpolarity["sentiment"] = df_getpolarity["post_id"].map(
                    dict(df_sentiment[SENTIMENT_COLUMNS].values)
                )
                df_getpolarity = df_getpolarity.dropna(
                    subset=["sentiment"]
                ).reset_index(drop=True)
                df_getpolarity["count_comments"] = [
                    sum(polarity.values()) for polarity in df_getpolarity["sentiment"]
                ]
                df_getpolarity = df_getpolarity[df_getpolarity["count_comments"] >= 10]
                if len(df_getpolarity) > 0:
                    df_getpolarity["rel_polarity"] = df_getpolarity.apply(
                        lambda row: np.dot(
                            list(row["sentiment"].keys()),
                            [
                                value / row["count_comments"]
                                for value in row["sentiment"].values()
                            ],
                        )
                    )
                    df_getpolarity = df_getpolarity[
                        df_getpolarity["rel_polarity"] > 0.1
                    ]
                    return (
                        df_getpolarity.sort_values(by=["rel_polarity"], ascending=False)
                        .reset_index(drop=True)
                        .head(threshold)
                    )
                else:
                    print(
                        "Warning: no post has more than 10 comments, thus can't be analyze"
                    )
                    return pd.DataFrame(columns=OUTPUT_COLUMNS)
            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                return pd.DataFrame(columns=OUTPUT_COLUMNS)
        else:
            print("Warning: One of the DataFrames is empty. It cannot be computed.")
            self.df_getpolarity = pd.DataFrame(columns=OUTPUT_COLUMNS)
            self.len_getpolarity = len(self.df_getpolarity)



import sys
from copy import deepcopy
from datetime import timedelta

import numpy as np
import pandas as pd

from wj_analysis.twitter import polarity_distribution

ERR_SYS = "\nSystem error: "


class PolarityAnalysisTW:
    def __init__(self, df_sentiment):
        self.df_sentiment = df_sentiment

    def top_post(self, df_replies, threshold=10):
        METHOD_NAME = "posts"

        REPLIES_COLUMNS = [
            "in_reply_to_screen_name",
            "in_reply_to_user_id",
            "text",
            "in_reply_to_status_id",
        ]
        SENTIMENT_COLUMNS = ["post_id", "sentiment"]

        OUTPUT_COLUMNS = [
            "name",
            "page_id",
            "post_id",
            "message",
            "permalink_url",
            "sentiment",
            "rel_sentiment",
        ]

        if len(self.df_sentiment) > 0 and len(df_replies) > 0:
            try:
                df_getpolarity = deepcopy(df_replies[REPLIES_COLUMNS])
                df_getpolarity = df_getpolarity.rename(
                    columns={
                        "in_reply_to_screen_name": "name",
                        "text": "message",
                        "in_reply_to_status_id": "post_id",
                        "in_reply_to_user_id": "page_id",
                    }
                )
                df_getpolarity["sentiment"] = df_getpolarity["post_id"].map(
                    dict(self.df_sentiment[SENTIMENT_COLUMNS].values)
                )
                df_getpolarity = df_getpolarity.dropna(
                    subset=["sentiment"]
                ).reset_index(drop=True)
                df_getpolarity["count_comments"] = [
                    sum(polarity.values()) for polarity in df_getpolarity["sentiment"]
                ]
                df_getpolarity = df_getpolarity[df_getpolarity["count_comments"] >= 10]
                if len(df_getpolarity) > 0:
                    rel_polarity = []
                    for index, row in df_getpolarity.iterrows():
                        for key in row["sentiment"].keys():
                            row["sentiment"][key] /= row["count_comments"]
                        rel_polarity.append(
                            np.dot(
                                list(row["sentiment"].keys()),
                                list(row["sentiment"].values()),
                            )
                        )
                    df_getpolarity.insert(2, "rel_polarity", rel_polarity, True)
                    df_getpolarity = df_getpolarity[
                        df_getpolarity["rel_polarity"] > 0.1
                    ]
                    df_getpolarity["permalink_url"] = [
                        "twitter.com/user/status/{}".format(text)
                        for text in df_getpolarity["post_id"]
                    ]
                    return (
                        df_getpolarity.sort_values(by=["rel_polarity"], ascending=False)
                        .reset_index(drop=True)
                        .head(threshold)
                    )
                else:
                    print(
                        "Warning: no post has more than 10 comments, thus can't be analyze"
                    )
                    return pd.DataFrame(columns=OUTPUT_COLUMNS)
            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                return pd.DataFrame(columns=OUTPUT_COLUMNS)
        else:
            print("Warning: One of the DataFrames is empty. It cannot be computed.")
            self.df_getpolarity = pd.DataFrame(columns=OUTPUT_COLUMNS)
            self.len_getpolarity = len(self.df_getpolarity)

    def timeaccount_post(self):
        METHOD_NAME = "timeaccount_post"

        OUTPUT_COLUMNS = ["_object_id", "_object_name", "date", "day_sentiment"]

        if len(self.df_sentiment) > 0:
            try:
                self.df_sentiment["created_time"] = pd.to_datetime(
                    self.df_sentiment["created_time"]
                ) - timedelta(hours=5)
                self.df_sentiment["date"] = self.df_sentiment["created_time"].apply(
                    lambda d: d.date()
                )
                df_timeaccount = pd.DataFrame(columns=OUTPUT_COLUMNS)
                page_names = dict(
                    self.df_sentiment[["_object_id", "_object_name"]].values
                )
                for brand_id in self.df_sentiment["_object_id"].drop_duplicates():
                    df_brand = self.df_sentiment[
                        self.df_sentiment["_object_id"] == brand_id
                    ]
                    for date in df_brand["date"].drop_duplicates():
                        df_date = df_brand[df_brand["date"] == date]
                        day_sentiment = {
                            -1.0: 0.0,
                            -0.5: 0.0,
                            0.0: 0.0,
                            0.5: 0.0,
                            1.0: 0.0,
                        }
                        for dict_values in df_date["sentiment"]:
                            for key in np.arange(-1.0, 1.5, 0.5):
                                day_sentiment[key] += dict_values[key]
                        df_timeaccount.loc[len(df_timeaccount)] = [
                            brand_id,
                            page_names[brand_id],
                            date,
                            day_sentiment,
                        ]
                df_timeaccount["count_comments"] = [
                    sum(polarity.values())
                    for polarity in df_timeaccount["day_sentiment"]
                ]
                if len(df_timeaccount) > 0:
                    for index, row in df_timeaccount.iterrows():
                        for key in row["day_sentiment"].keys():
                            row["day_sentiment"][key] /= row["count_comments"]
                return df_timeaccount
            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                return pd.DataFrame(columns=OUTPUT_COLUMNS)
        else:
            print("Warning: One of the DataFrames is empty. It cannot be computed.")
            self.df_getpolarity = pd.DataFrame(columns=OUTPUT_COLUMNS)
            self.len_getpolarity = len(self.df_getpolarity)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul  7 20:30:53 2021

@author: oscar
"""

import traceback
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None


ERR_SYS = "system error: "
CLASS_NAME = "DemographicKpi"
GENDERS = ["F", "M", "U"]


def cal_perc(df_cp, round_data):
    """
    This function calculate percent a column, create a percent column

    Parameters
    ----------
    df_cp : TYPE dataframe
        DESCRIPTION. dataframe with number column
    round_data : TYPE int
        DESCRIPTION. number of decimals

    Returns
    -------
    Create percent column

    """
    df_cp["percent"] = (df_cp["value"] / sum(df_cp["value"])) * 100
    df_cp["percent"] = round(df_cp["percent"], round_data)


def extract_age_gender(df_et_ag, column_ag="indicator"):
    """
    This function extract age and gender

    Parameters
    ----------
    df_et_ag : TYPE dataframe
        DESCRIPTION. dataframe with indicator column to calculate
    column_ag : TYPE, string optional
        DESCRIPTION. The default is 'indicator'.

    Returns
    -------
    df_et_ag : TYPE dataframe
        DESCRIPTION. dataframe with gender and age column

    """
    try:
        df_et_ag["gender"] = df_et_ag[column_ag].apply(lambda x: x[0])
        df_et_ag["age"] = df_et_ag[column_ag].apply(lambda x: x[2:])

    except TypeError as e_1:
        print("".center(60, "="))
        print(e_1)
        print("".center(60, "="))
        print(ERR_SYS + str(exc_info()[0]))
        print("".center(60, "="))
        print(f"Class: {CLASS_NAME}")
        print("".center(60, "="))
        traceback.print_exc()

        df_et_ag["gender"] = float("nan")
        df_et_ag["age"] = float("nan")

    return df_et_ag


def extract_period(df_dem, column="end_time"):
    """
    This function extract the period to analyze and the dates

    Parameters
    ----------
    df_dem : TYPE dataframe
        DESCRIPTION. dataframe insights demographics
    social_n : TYPE string
        DESCRIPTION. social net 'fb' Facebook, 'ig' Instagram

    Returns
    -------
    column : TYPE string
        DESCRIPTION. column date
    """

    col_ag = ["indicator", "value", "date"]

    df_dem[column] = pd.to_datetime(df_dem[column])
    df_dem["date"] = df_dem[column].apply(lambda x: x.date)
    date_max = max(df_dem["date"])

    df_dem = df_dem[col_ag]
    df_dem = df_dem[df_dem["date"] == date_max].reset_index(drop=True)

    return df_dem


class DemographicKpi:
    """
    This class calculates the insights data for visualizations in Oceana
    """

    def __init__(self, social_n):

        self.social_n = social_n

    def age_gender(self, df_ag, round_d=1):
        """
        This function calculate age and gender

        Parameters
        ----------
        df_ag : TYPE dataframe
            DESCRIPTION. dataframe with insights data
        round_d : TYPE, optionaint
            DESCRIPTION. The default is 1. decimals in percent

        Returns
        -------
        df_gender : TYPE dataframe
            DESCRIPTION. dataframe with data gender
        max_gender : TYPE list
            DESCRIPTION. list with max value gender
        list_age : TYPE list with daframes
            DESCRIPTION. list_age[0], age data
                         list_age[1], gender 'F' group by age
                         list_age[2], gender 'M' group by age
                         list_age[3], gender 'U' group by age

        """

        social_n = self.social_n

        try:

            if social_n == "fb":
                df_ag_n = df_ag
            elif social_n == "ig":
                df_ag_n = df_ag[df_ag["name"] == "audience_gender_age"]
            else:
                print("".center(60, "="))
                print(f"=>{social_n}<= !!!!!!! no valid social network")
                print("".center(60, "="))

            df_ag_n = extract_period(df_dem=df_ag_n)

            df_gender = extract_age_gender(df_et_ag=df_ag_n)

            df_gender = df_gender.groupby("gender", as_index=False).sum()
            df_gender = df_gender.sort_values("value", ascending=False).reset_index(
                drop=True
            )
            cal_perc(df_gender, round_d)
            max_gender = df_gender[df_gender["value"] == max(df_gender["value"])]
            max_gender[["value", "percent"]] = max_gender[["value", "percent"]].astype(
                float
            )
            if max_gender["gender"].iloc[0] == "M":
                max_gender["gender"] = "Hombres"
            elif max_gender["gender"].iloc[0] == "F":
                max_gender["gender"] = "Mujeres"
            elif max_gender["gender"].iloc[0] == "U":
                max_gender["gender"] = "No determinado"
            max_gender = dict(max_gender.iloc[0])

            df_age = df_ag_n.groupby(["gender", "age"], as_index=False).sum()
            df_age_all = df_age.groupby("age", as_index=False).sum()
            df_age_all = df_age_all.sort_values("value", ascending=False)
            cal_perc(df_age_all, round_data=round_d)

            df_age_all["gender"] = "Group"

            max_age = df_age[df_age["value"] == max(df_age["value"])]
            max_age["percent"] = round(
                (max(df_age["value"]) / sum(df_age["value"])) * 100, round_d
            )
            max_age[["value", "percent"]] = max_age[["value", "percent"]].astype(float)
            if max_age["gender"].iloc[0] == "M":
                max_age["gender"] = "Hombres"
            elif max_age["gender"].iloc[0] == "F":
                max_age["gender"] = "Mujeres"
            elif max_age["gender"].iloc[0] == "U":
                max_age["gender"] = "No determinado"
            max_age = dict(max_age.iloc[0])

            for g_e in GENDERS:
                df_g = df_age[df_age["gender"] == g_e]
                df_g = df_g.sort_values("value", ascending=False).reset_index(drop=True)
                cal_perc(df_cp=df_g, round_data=round_d)
                df_age_all = pd.concat([df_age_all, df_g], sort=True)

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_gender = pd.DataFrame(columns=["gender", "value", "percent"])
            max_gender = dict()
            df_age_all = pd.DataFrame(columns=["age", "gender", "percent", "value"])
            max_age = dict()

        except AttributeError as e_2:
            print("".center(60, "="))
            print(e_2)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_gender = pd.DataFrame(columns=["gender", "value", "percent"])
            max_gender = dict()
            df_age_all = pd.DataFrame(columns=["age", "gender", "percent", "value"])
            max_age = dict()

        except ValueError as e_3:
            print("".center(60, "="))
            print(e_3)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_gender = pd.DataFrame(columns=["gender", "value", "percent"])
            max_gender = dict()
            df_age_all = pd.DataFrame(columns=["age", "gender", "percent", "value"])
            max_age = dict()

        return df_gender, max_gender, df_age_all, max_age

    def city(self, df_ci, round_d=1, top_bar=7, top_bur=3):
        """
        this function calculate data of city from insights

        Parameters
        ----------
        df_ci : TYPE dataframe
            DESCRIPTION. dataframe with data city insights
        round_d : TYPE, optional int
            DESCRIPTION. The default is 1. decimal percent
        top_bar : TYPE, optional int
            DESCRIPTION. The default is 7. top cities display barplot
        top_bur : TYPE, optional int
            DESCRIPTION. The default is 3. for the display of bubbles
            the number of bubbles to be displayed

        Returns
        -------
        df_ci_burplot : TYPE dataframe
            DESCRIPTION. data bubbles plot
        df_ci_bur : TYPE dataframe
            DESCRIPTION. data barplot

        """

        social_n = self.social_n

        try:

            if social_n == "fb":
                df_ci_n = df_ci
            elif social_n == "ig":
                df_ci_n = df_ci[df_ci["name"] == "audience_city"]
            else:
                print("".center(60, "="))
                print(f"=>{social_n}<= !!!!!!! no valid social network")
                print("".center(60, "="))

            df_ci_n = extract_period(df_dem=df_ci_n)

            df_ci_n["city"] = df_ci_n["indicator"].apply(lambda x: x.split(",")[0])
            cal_perc(df_ci_n, round_d)
            df_ci_n = df_ci_n[["value", "city", "percent"]]

            df_ci_n = df_ci_n.sort_values("value", ascending=False).reset_index(
                drop=True
            )

            df_ci_bur = df_ci_n[0 : (top_bur - 1)]
            others = dict(df_ci_n[top_bur - 1 :].sum())
            hover = df_ci_n["city"][top_bur - 1 :].tolist()
            df_ci_bur = (
                df_ci_bur.append(
                    {
                        "value": others.get("value"),
                        "city": "Otros",
                        "hover": hover,
                        "percent": round(others.get("percent"), round_d),
                    },
                    ignore_index=True,
                )
            ).fillna(0)

            df_ci_bar_plot = df_ci_n.iloc[0 : top_bar - 1]
            others_bar = dict(df_ci_n[top_bar - 1 :].sum())
            hover_bar = df_ci_n["city"][top_bar - 1 :].tolist()
            df_ci_bar_plot = (
                df_ci_bar_plot.append(
                    {
                        "value": others_bar.get("value"),
                        "city": "Otros",
                        "hover": hover_bar,
                        "percent": round(others_bar.get("percent"), round_d),
                    },
                    ignore_index=True,
                )
            ).fillna(0)

            max_city = df_ci_n[df_ci_n["value"] == max(df_ci_n["value"])]
            max_city[["value", "percent"]] = max_city[["value", "percent"]].astype(
                float
            )
            max_city = dict(max_city.iloc[0])

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_ci_bar_plot = pd.DataFrame(columns=["city", "value", "percent"])
            df_ci_bur = pd.DataFrame(columns=["city", "value", "percent"])
            max_city = dict()

        except AttributeError as e_2:
            print("".center(60, "="))
            print(e_2)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_ci_bar_plot = pd.DataFrame(columns=["city", "value", "percent"])
            df_ci_bur = pd.DataFrame(columns=["city", "value", "percent"])
            max_city = dict()

        except ValueError as e_3:
            print("".center(60, "="))
            print(e_3)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_ci_bar_plot = pd.DataFrame(columns=["city", "value", "percent"])
            df_ci_bur = pd.DataFrame(columns=["city", "value", "percent"])
            max_city = dict()

        return df_ci_bar_plot, df_ci_bur, max_city

    def like_source(self, df_like_s, round_d=1):
        """
        this function is only implemented for Facebook

        Parameters
        ----------
        df_like_s : TYPE dataframe
            DESCRIPTION. dataframe whit data of ads Facebok
        round_d : TYPE, optional int
            DESCRIPTION. The default is 1. decimals of percent

        Returns
        -------
        df_like_s : TYPE dataframe
            DESCRIPTION. dataframe with data ads
        max_like_s : TYPE list
            DESCRIPTION. max value ads

        """

        social_n = self.social_n

        if social_n == "fb":
            pass
        elif social_n == "ig":
            print("Not implemented for Instagram yet")
        else:
            print("".center(60, "="))
            print(f"=>{social_n}<= !!!!!!! no valid social network")
            print("".center(60, "="))

        try:

            names_org = [
                "Ads",
                "News Feed",
                "Page Suggestions",
                "Search",
                "Your Page",
                "Other",
                "Restored Likes from Reactivated Accounts",
            ]
            names_new = [
                "Ads",
                "Feed",
                "Sugeridas",
                "Búsqueda",
                "Directos",
                "Otros",
                "Otros",
            ]

            dict_names = dict(zip(names_org, names_new))

            df_like_s = extract_period(df_dem=df_like_s)
            df_like_s["indicator"] = df_like_s["indicator"].replace(dict_names)

            df_like_s = df_like_s.groupby("indicator", as_index=False).sum()

            cal_perc(df_like_s, round_data=round_d)
            df_like_s = df_like_s.sort_values("value", ascending=False)

            max_like_s = df_like_s[df_like_s["value"] == max(df_like_s["value"])]
            max_like_s = list(max_like_s.iloc[0])

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_like_s = pd.DataFrame(columns=["indicator", "value", "percent"])
            max_like_s = list()

        except AttributeError as e_2:
            print("".center(60, "="))
            print(e_2)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_like_s = pd.DataFrame(columns=["indicator", "value", "percent"])
            max_like_s = list()

        except ValueError as e_3:
            print("".center(60, "="))
            print(e_3)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME}")
            print("".center(60, "="))
            traceback.print_exc()

            df_like_s = pd.DataFrame(columns=["indicator", "value", "percent"])
            max_like_s = list()

        return df_like_s, max_like_s



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun  2 15:22:27 2021

@author: oscar
"""

import ast
import traceback
from sys import exc_info

import pandas as pd

pd.options.mode.chained_assignment = None


ERR_SYS = "system error: "

COL_POST_F = [
    "page_id",
    "post_id",
    "post_from",
    "permalink_url",
    "shares",
    "likes",
    "comments",
    "reactions_love",
    "reactions_wow",
    "reactions_haha",
    "created_time",
    "reactions_sad",
    "reactions_angry",
    "reactions_thankful",
    "full_picture",
]

COL_INS_F = [
    "post_id",
    "post_impressions",
    "post_impressions_unique",
    "post_impressions_paid_unique",
    "post_impressions_organic_unique",
]

COL_POST_I = [
    "account_id",
    "owner_username",
    "shortcode",
    "date_local",
    "url",
    "likes_count",
    "comment_count",
]

COL_INS_I = ["name", "period", "value", "end_time", "shortcode"]

METHOD_NAME = ["reach", "interact", "impressions", "engagement_rate", "ts_reach"]

CLASS_NAME = "insights_kpi"

# flags with cases in compare periods
FLAGS = ["n==b", "n>b", "n<b", "n<period_and_b==0", "b==0", "except"]


def select_last_post(df_p, id_name, id_last):
    """
    This function extract the last record in the database
    with equal identifier

    Parameters
    ----------
    df_p : TYPE, dataframe
        DESCRIPTION. dataframe with post identifier
    id_name : TYPE string
        DESCRIPTION. post identifier
    id_last : TYPE string
        DESCRIPTION. columns to max select

    Returns
    -------
    df_post : TYPE dataframe
        DESCRIPTION. dataframe with last record

    """
    df_post_temp = pd.DataFrame()
    for i in df_p[id_name].unique():
        temp1 = df_p[df_p[id_name] == i]
        temp2 = max(temp1[id_last])
        temp1 = temp1[temp1[id_last] == temp2]
        df_post_temp = pd.concat([df_post_temp, temp1])
    if len(df_post_temp) == 0:
        df_post = pd.DataFrame(columns=df_p.columns)
    else:
        df_post = df_post_temp

    return df_post


def dataframes_ts(df_r1, df_r2, date_max, days, column_reach):
    """
    This function generates the time series and makes a cumulative sum of the data

    Parameters
    ----------
    df_r1 : TYPE dataframe
        DESCRIPTION. with first period
    df_r2 : TYPE dataframe
        DESCRIPTION. with second period
    date_max : TYPE datetime.date
        DESCRIPTION. date incial to analice
    days : TYPE int
        DESCRIPTION. period to analice
    column_reach : TYPE str
        DESCRIPTION. name of colum to analice

    Returns
    -------
    df_ts1 : TYPE dataframe
        DESCRIPTION. with first period with time serie
    df_ts2 : TYPE dataframe
        DESCRIPTION. with second period with time serie

    """

    date_range1 = []
    for day in range(days):
        date_range1.append(date_max - pd.to_timedelta(day, unit="d"))

    date_range2 = []
    for day in range(days):
        date_range2.append(
            (date_range1[-1] - pd.to_timedelta(1, unit="d"))
            - pd.to_timedelta(day, unit="d")
        )

    if len(df_r1) > 0:
        df_r1 = df_r1.groupby("date", as_index=False).sum()

    if len(df_r2) > 0:
        df_r2 = df_r2.groupby("date", as_index=False).sum()

    df_ts1 = pd.DataFrame(columns=["date", "reach", "period", "pos"])
    df_ts2 = pd.DataFrame(columns=["date", "reach", "period", "pos"])

    date_range1 = list(reversed(date_range1))
    date_range2 = list(reversed(date_range2))

    df_ts1["date"] = date_range1
    it_d = 0
    for day1 in date_range1:
        df_ts1["date"].iloc[it_d] = day1
        df_ts1["period"].iloc[it_d] = "now"
        df_ts1["pos"].iloc[it_d] = it_d
        df_temp1 = df_r1[df_r1["date"] == day1]
        if len(df_temp1) > 0:
            df_ts1["reach"].iloc[it_d] = df_temp1[column_reach].iloc[0]
        else:
            df_ts1["reach"].iloc[it_d] = 0
        it_d += 1

    df_ts2["date"] = date_range2
    it_d = 0
    for day2 in date_range2:
        df_ts2["date"].iloc[it_d] = day2
        df_ts2["period"].iloc[it_d] = "before"
        df_ts2["pos"].iloc[it_d] = it_d
        df_temp2 = df_r2[df_r2["date"] == day2]
        if len(df_temp2) > 0:
            df_ts2["reach"].iloc[it_d] = df_temp2[column_reach].iloc[0]
        else:
            df_ts2["reach"].iloc[it_d] = 0
        it_d += 1

    return df_ts1, df_ts2


def delta(data1, data2, rou=1):
    """
    delta calculate percentage difference between two numbers

    Parameters
    ----------
    data1 : TYPE. int or float
        DESCRIPTION. first number
    data2 : TYPE int or float
        DESCRIPTION. second number
    rou : TYPE, optional int
        DESCRIPTION. The default is 1. roun doutput

    Returns
    -------
    dta : TYPE float
        DESCRIPTION. percentage difference between two numbers

    """

    if data2 > 0:
        dta = (1 - (data1 / data2)) * 100
        dta = round(dta, rou)
    else:
        dta = 0

    return dta


def reactions_cal(df_reach, social_n):
    """
    This function sums the reactions

    Parameters
    ----------
    df_reach : TYPE dataframe
        DESCRIPTION. dataframe whit Facebook or Instagram columns
    social_n : TYPE string
        DESCRIPTION. social network

    Returns
    -------
    all_reactions : TYPE dict
        DESCRIPTION. dict with the sum of the reactions
    total_reactions : TYPE
        DESCRIPTION. sum total reactions

    """
    if social_n == "fb":

        all_reactions = {
            "total_shares": sum(df_reach["shares"]),
            "total_likes": sum(df_reach["likes"]),
            "total_comments": sum(df_reach["comments"]),
            "total_love": sum(df_reach["reactions_love"]),
            "total_wow": sum(df_reach["reactions_wow"]),
            "total_haha": sum(df_reach["reactions_haha"]),
            "total_sad": sum(df_reach["reactions_sad"]),
            "total_angry": sum(df_reach["reactions_angry"]),
            "total_thankful": sum(df_reach["reactions_thankful"]),
        }

    else:
        all_reactions = {
            "total_saved": sum(df_reach["saved"]),
            "total_video_views": sum(df_reach["video_views"]),
            "total_likes_count": sum(df_reach["likes_count"]),
            "total_comment_count": sum(df_reach["comment_count"]),
        }

    reactions = []
    for reaction in all_reactions:
        reactions.append(all_reactions.get(reaction))

    total_reactions = sum(reactions)

    return all_reactions, total_reactions


def extrac_period(df_no_dt, days, social_n):
    """
    This function extract the period to analyze and the dates

    Parameters
    ----------
    df_no_dt : TYPE dataframe
        DESCRIPTION. dataframe whith post information
    days : TYPE int
        DESCRIPTION. period in days to analice
    social_n : TYPE str only 'fb' or 'ig'
        DESCRIPTION. 'fb' to Facebook, 'ig' to Instagram,

    Returns
    -------
    df_period_now : TYPE dataframe
        DESCRIPTION. filtered data of the current period
    df_period_bef : TYPE dataframe
        DESCRIPTION. filtered data from the previous period
    flag : TYPE str
        DESCRIPTION. indicates the behavior of the periods, as follows:
            'n==b': periods have the same amount of data data
            'n>b': the first period has more data than the second
            'n<b': the second period has more data than the first
            'n<period and b==0': the first period has less data than
            those consulted and the second has no data
            'b==0': the second perdiod has no data
            'except': an exception occurred
    delta_time : TYPE date
        DESCRIPTION. list with date range

    """

    if social_n == "fb":
        column = "created_time"
        col_post = COL_POST_F

    elif social_n == "ig":
        column = "date_local"
        col_post = COL_POST_I
    else:
        print(f"{social_n} no valid social network")

    df_period_now_empty = pd.DataFrame(columns=col_post)
    df_period_before_empty = pd.DataFrame(columns=col_post)

    try:

        df_no_dt[column] = pd.to_datetime(df_no_dt[column])
        df_no_dt["date"] = df_no_dt[column].apply(lambda x: x.date)

        period = pd.to_timedelta(days, unit="d")

        date_max1 = max(df_no_dt["date"])
        date_min1 = date_max1 - (period - pd.to_timedelta(1, unit="d"))
        date_max2 = date_min1 - pd.to_timedelta(1, unit="d")
        date_min2 = date_max2 - period

        df_period_now = df_no_dt[
            (df_no_dt["date"] <= date_max1) & (df_no_dt["date"] >= date_min1)
        ]
        df_period_bef = df_no_dt[
            (df_no_dt["date"] <= date_max2) & (df_no_dt["date"] >= date_min2)
        ]

        df_period_now = df_period_now.drop_duplicates()
        df_period_bef = df_period_bef.drop_duplicates()

        if (len(df_period_now) == len(df_period_bef)) and (len(df_period_bef) != 0):
            flag = FLAGS[0]
        elif (len(df_period_now) > len(df_period_bef)) and (len(df_period_bef) != 0):
            flag = FLAGS[1]
        elif (len(df_period_now) < len(df_period_bef)) and (len(df_period_bef) != 0):
            flag = FLAGS[2]
        elif (len(df_period_now) < period.days) and (len(df_period_bef) == 0):
            flag = FLAGS[3]
        elif len(df_period_bef) == 0:
            flag = FLAGS[4]

        delta_time = [date_max1, date_min1, date_max2, date_min2]

    except KeyError as e_1:
        print("".center(60, "="))
        print(e_1)
        print("".center(60, "="))
        print(ERR_SYS + str(exc_info()[0]))
        print("".center(60, "="))
        print(f"Class: {CLASS_NAME}")
        print("".center(60, "="))
        traceback.print_exc()

        df_period_now = df_period_now_empty
        df_period_bef = df_period_before_empty
        flag = FLAGS[5]
        delta_time = ["", "", "", ""]

    except AttributeError as e_2:
        print("".center(60, "="))
        print(e_2)
        print("".center(60, "="))
        print(ERR_SYS + str(exc_info()[0]))
        print("".center(60, "="))
        print(f"Class: {CLASS_NAME}")
        print("".center(60, "="))
        traceback.print_exc()

        df_period_now = df_period_now_empty
        df_period_bef = df_period_before_empty
        flag = FLAGS[5]
        delta_time = ["", "", "", ""]

    except ValueError as e_3:
        print("".center(60, "="))
        print(e_3)
        print("".center(60, "="))
        print(ERR_SYS + str(exc_info()[0]))
        print("".center(60, "="))
        print(f"Class: {CLASS_NAME}")
        print("".center(60, "="))
        traceback.print_exc()

        df_period_now = df_period_now_empty
        df_period_bef = df_period_before_empty
        flag = "except"
        delta_time = ["", "", "", ""]

    return df_period_now, df_period_bef, flag, delta_time


def extract_name(df_post, column="name"):
    """
    extract_name of the facebook brand to analyze

    Parameters
    ----------
    df_post : TYPE dataframe
        DESCRIPTION. datafraem whith post
    column : TYPE, optional
        DESCRIPTION. The default is 'name', for extract id 'id'

    Returns
    -------
    df : TYPE dataframe
        DESCRIPTION. dataframe whit column parameter name

    """

    try:
        brand_name = df_post["post_from"].apply(ast.literal_eval)
        id_column = []
        for i in range(len(df_post)):
            temp_1 = brand_name.iloc[i][column]
            id_column.append(temp_1)
        df_post[column] = id_column
        df_post.drop(columns="post_from", inplace=True)

    except KeyError as e_1:
        print("".center(60, "="))
        print(e_1)
        print("".center(60, "="))
        print(ERR_SYS + str(exc_info()[0]))
        print("".center(60, "="))
        print(f"Class: {CLASS_NAME}")
        print("".center(60, "="))
        traceback.print_exc()

        df_post[column] = "no_name"

    return df_post


class InsightsKpi:
    """
    This class calculates the information needed for the kpi for
    the Facebook and Instagram reach dashboard.
    """

    def __init__(self, df_p, df_i, days, social_n):
        """
        This function prepare data

        Parameters
        ----------
        df_p : TYPE dataframe
            DESCRIPTION. dataframe whith post information
        df_i : TYPE dataframe
            DESCRIPTION. dataframe whith insights information
        days : TYPE int
            DESCRIPTION. period in days to analice
        social_n : TYPE str only 'fb' or 'ig'
            DESCRIPTION. 'fb' to Facebook, 'ig' to Instagram,

        Returns
        -------
        None.

        """

        if social_n == "fb":
            df_p = extract_name(df_post=df_p, column="name")
            column = "post_id"
        elif social_n == "ig":
            column = "shortcode"
            df_p = select_last_post(df_p=df_p, id_name="shortcode", id_last="id")
        else:
            print("".center(60, "="))
            print(f"=>{social_n}<= !!!!!!! no valid social network")
            print("".center(60, "="))
            column = ""

        self.social_n = social_n

        self.days = days

        if days <= 1:
            print("the minimum period is two days")
            days = 2

        df_all = df_p.merge(df_i, on=column, how="outer")
        self.df_1, self.df_2, self.flag, self.d_time = extrac_period(
            df_no_dt=df_all, days=days, social_n=social_n
        )
        self.df_1 = self.df_1.fillna(0)
        self.df_2 = self.df_2.fillna(0)

        self.dates = {
            "date_max1": self.d_time[0],
            "date_min1": self.d_time[1],
            "date_max2": self.d_time[2],
            "date_min2": self.d_time[3],
        }

    def reach(self):
        """
        This function calculate reac

        Returns
        -------
        reach : TYPE dict
            DESCRIPTION. information reach

        """

        social_n = self.social_n
        reach_date_max1 = self.dates.get("date_max1")
        reach_date_min1 = self.dates.get("date_min1")
        reach_date_max2 = self.dates.get("date_max2")
        reach_date_min2 = self.dates.get("date_min2")

        if social_n == "fb":
            col_reach = [
                "date",
                "post_impressions_unique",
                "post_impressions_organic_unique",
                "post_impressions_paid_unique",
            ]
        else:
            col_reach = ["date", "impressions", "reach"]

        try:
            df_reach1 = self.df_1[col_reach]
            df_reach2 = self.df_2[col_reach]

            # Reach facebook
            if social_n == "fb":
                reach = sum(df_reach1["post_impressions_unique"])
                if self.flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    reach_delta = 0
                else:
                    reach_delta = delta(
                        reach, sum(df_reach2["post_impressions_unique"])
                    )

                reach_organic = sum(df_reach1["post_impressions_organic_unique"])
                reach_paid = sum(df_reach1["post_impressions_paid_unique"])

                reach = {
                    "reach": reach,
                    "reach_delta": abs(reach_delta),
                    "reach_organic": reach_organic,
                    "reach_paid": reach_paid,
                    "reach_date_max1": reach_date_max1,
                    "reach_date_min1": reach_date_min1,
                    "reach_date_max2": reach_date_max2,
                    "reach_date_min2": reach_date_min2,
                    "flag": self.flag,
                }

            # Reach Instagram
            elif social_n == "ig":
                reach = sum(df_reach1["reach"])
                if self.flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    reach_delta = 0
                else:
                    reach_delta = delta(reach, sum(df_reach2["reach"]))

                reach_impressions = sum(df_reach1["impressions"])

                reach = {
                    "reach": reach,
                    "reach_delta": abs(reach_delta),
                    "reach_impressions": reach_impressions,
                    "reach_date_max1": reach_date_max1,
                    "reach_date_min1": reach_date_min1,
                    "reach_date_max2": reach_date_max2,
                    "reach_date_min2": reach_date_min2,
                    "flag": self.flag,
                }

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME} => {METHOD_NAME[0]}")
            print("".center(60, "="))
            traceback.print_exc()

            if social_n == "fb":
                reach = {
                    "reach": "",
                    "reach_delta": "",
                    "reach_organic": "",
                    "reach_paid": "",
                    "reach_date_max1": "",
                    "reach_date_min1": "",
                    "reach_date_max2": "",
                    "reach_date_min2": "",
                    "flag": self.flag,
                }
            else:
                reach = {
                    "reach": "",
                    "reach_delta": "",
                    "reach_impressions": "",
                    "reach_date_max1": "",
                    "reach_date_min1": "",
                    "reach_date_max2": "",
                    "reach_date_min2": "",
                    "flag": self.flag,
                }

        return reach

    def interact(self):
        """
        This function calculate interact (likes, share ...)

        Returns
        -------
        inter : TYPE dict
            DESCRIPTION. information interact

        """

        social_n = self.social_n

        if social_n == "fb":
            col_inter = [
                "date",
                "shares",
                "likes",
                "comments",
                "reactions_love",
                "reactions_wow",
                "reactions_haha",
                "reactions_sad",
                "reactions_angry",
                "reactions_thankful",
            ]
        else:
            col_inter = ["date", "likes_count", "comment_count", "saved", "video_views"]

        try:
            # Interact Facebook
            if social_n == "fb":

                all_inter1, total_inter1 = reactions_cal(
                    self.df_1[col_inter], social_n=social_n
                )
                _, total_inter2 = reactions_cal(self.df_2[col_inter], social_n=social_n)

                if self.flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    inter_delta = 0
                else:
                    inter_delta = delta(total_inter1, total_inter2)

                inter = {
                    "interact": total_inter1,
                    "inter_delta": abs(inter_delta),
                    "inter_likes": all_inter1.get("total_likes"),
                    "inter_shares": all_inter1.get("total_shares"),
                    "inter_comments": all_inter1.get("total_comments"),
                    "inter_date_max1": self.dates.get("date_max1"),
                    "inter_date_min1": self.dates.get("date_min1"),
                    "inter_date_max2": self.dates.get("date_max2"),
                    "inter_date_min2": self.dates.get("date_min2"),
                    "flag": self.flag,
                }

            # Interact Instagram
            elif social_n == "ig":
                all_inter1, total_inter1 = reactions_cal(
                    self.df_1[col_inter], social_n=social_n
                )
                _, total_inter2 = reactions_cal(self.df_2[col_inter], social_n=social_n)

                if self.flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    inter_delta = 0
                else:
                    inter_delta = delta(total_inter1, total_inter2)

                inter = {
                    "interact": total_inter1,
                    "inter_delta": abs(inter_delta),
                    "inter_likes": all_inter1.get("total_likes_count"),
                    "inter_comment": all_inter1.get("total_comment_count"),
                    "inter_saved": all_inter1.get("total_saved"),
                    "inter_video_views": all_inter1.get("total_video_views"),
                    "inter_date_max1": self.dates.get("date_max1"),
                    "inter_date_min1": self.dates.get("date_min1"),
                    "inter_date_max2": self.dates.get("date_max2"),
                    "inter_date_min2": self.dates.get("date_min2"),
                    "flag": self.flag,
                }

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME} => {METHOD_NAME[1]}")
            print("".center(60, "="))
            traceback.print_exc()

            if social_n == "fb":
                inter = {
                    "interact": "",
                    "inter_delta": "",
                    "inter_likes": "",
                    "inter_shares": "",
                    "inter_comments": "",
                    "inter_date_max1": "",
                    "inter_date_min1": "",
                    "inter_date_max2": "",
                    "inter_date_min2": "",
                    "flag": self.flag,
                }
            else:
                inter = {
                    "interact": "",
                    "inter_delta": "",
                    "inter_likes": "",
                    "inter_comment": "",
                    "inter_saved_count": "",
                    "inter_video_views": "",
                    "inter_date_max1": "",
                    "inter_date_min1": "",
                    "inter_date_max2": "",
                    "inter_date_min2": "",
                    "flag": self.flag,
                }

        return inter

    def impressions(self):
        """
        This function calculate impressions

        Returns
        -------
        impress : TYPE dict
            DESCRIPTION. information impressions

        """

        flag = self.flag
        social_n = self.social_n
        impress_date_max1 = self.dates.get("date_max1")
        impress_date_min1 = self.dates.get("date_min1")
        impress_date_max2 = self.dates.get("date_max2")
        impress_date_min2 = self.dates.get("date_min2")

        if social_n == "fb":
            col_inter = ["date", "post_impressions"]
        else:
            col_inter = ["date", "impressions"]

        try:
            df_reach1 = self.df_1[col_inter]
            df_reach2 = self.df_2[col_inter]

            # Impressions Facebook
            if social_n == "fb":
                impressions = sum(df_reach1["post_impressions"])
                impressions2 = sum(df_reach2["post_impressions"])

                if flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    impress_delta = 0
                else:
                    impress_delta = delta(impressions, impressions2)

                impress = {
                    "impressions": impressions,
                    "impress_delta": abs(impress_delta),
                    "impress_date_max1": impress_date_max1,
                    "impress_date_min1": impress_date_min1,
                    "impress_date_max2": impress_date_max2,
                    "impress_date_min2": impress_date_min2,
                    "flag": flag,
                }

            # Impressions Instagram
            elif social_n == "ig":
                impressions = sum(df_reach1["impressions"])
                impressions2 = sum(df_reach2["impressions"])

                if flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    impress_delta = 0
                else:
                    impress_delta = delta(impressions, impressions2)

                impress = {
                    "impressions": impressions,
                    "impress_delta": abs(impress_delta),
                    "impress_date_max1": impress_date_max1,
                    "impress_date_min1": impress_date_min1,
                    "impress_date_max2": impress_date_max2,
                    "impress_date_min2": impress_date_min2,
                    "flag": flag,
                }

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME} => {METHOD_NAME[2]}")
            print("".center(60, "="))
            traceback.print_exc()

            impress = {
                "impressions": "",
                "impress_delta": "",
                "impress_date_max1": "",
                "impress_date_min1": "",
                "impress_date_max2": "",
                "impress_date_min2": "",
                "flag": flag,
            }

        return impress

    def engagement_rate(self):
        """
        This function calculate engagement_rate

        Returns
        -------
        engagement_rate : TYPE dict
            DESCRIPTION. information engagement_rate

        """

        flag = self.flag
        social_n = self.social_n

        if social_n == "fb":
            col_inter = [
                "date",
                "shares",
                "likes",
                "comments",
                "reactions_love",
                "reactions_wow",
                "reactions_haha",
                "reactions_sad",
                "reactions_angry",
                "reactions_thankful",
                "post_impressions_unique",
            ]
        else:
            col_inter = [
                "date",
                "likes_count",
                "comment_count",
                "saved",
                "video_views",
                "reach",
            ]

        try:
            df_reach1 = self.df_1[col_inter]
            df_reach2 = self.df_2[col_inter]

            # Engagement Facebook
            if social_n == "fb":
                _, total_inter1 = reactions_cal(df_reach1, social_n=social_n)
                _, total_inter2 = reactions_cal(df_reach2, social_n=social_n)

                if (
                    sum(df_reach1["post_impressions_unique"]) != 0
                    and sum(df_reach2["post_impressions_unique"]) != 0
                ):
                    eng = (
                        total_inter1 / sum(df_reach1["post_impressions_unique"])
                    ) * 100
                    eng2 = (
                        total_inter2 / sum(df_reach2["post_impressions_unique"])
                    ) * 100
                else:
                    eng = 0
                    eng2 = 0

                if flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    eng_delta = 0
                else:
                    eng_delta = delta(eng, eng2)

                engagement_rate = {
                    "engagement_rate": round(eng, 1),
                    "delta_eng": abs(eng_delta),
                    "flag": flag,
                }

            # Engagement Instagram
            elif social_n == "ig":
                _, total_inter1 = reactions_cal(df_reach1, social_n=social_n)
                _, total_inter2 = reactions_cal(df_reach2, social_n=social_n)

                if sum(df_reach1["reach"]) != 0 and sum(df_reach2["reach"]) != 0:
                    eng = (total_inter1 / sum(df_reach1["reach"])) * 100
                    eng2 = (total_inter2 / sum(df_reach2["reach"])) * 100
                else:
                    eng = 0
                    eng2 = 0

                if flag in (FLAGS[4], FLAGS[3], FLAGS[5]):
                    eng_delta = 0
                else:
                    eng_delta = delta(eng, eng2)

                engagement_rate = {
                    "engagement_rate": round(eng, 1),
                    "delta_eng": abs(eng_delta),
                    "flag": flag,
                }

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME} => {METHOD_NAME[3]}")
            print("".center(60, "="))
            traceback.print_exc()

            engagement_rate = {"engagement_rate": "", "delta_eng": "", "flag": flag}

        return engagement_rate

    def ts_reach(self, all_df=True):
        """
        This function generate time series with period to analyse

        Parameters
        ----------
        all_df : TYPE, optional
            DESCRIPTION. The default is True. If it is false,
            it only generates the time series of the current period.

        Returns
        -------
        df_all_reach : TYPE dataframe
            DESCRIPTION. dataframe with time serie selected

        """

        days = self.days
        social_n = self.social_n

        try:

            date_max1 = self.dates.get("date_max1")

            if social_n == "fb":
                column_reach_sn = "post_impressions_unique"
            else:
                column_reach_sn = "reach"

            col_inter = ["date", column_reach_sn]

            df_reach1 = self.df_1[col_inter]
            df_reach2 = self.df_2[col_inter]

            if all_df:
                df_ts1, df_ts2 = dataframes_ts(
                    df_r1=df_reach1,
                    df_r2=df_reach2,
                    date_max=date_max1,
                    days=days,
                    column_reach=column_reach_sn,
                )
                df_ts1["reach"] = df_ts1["reach"].cumsum()
                df_ts2["reach"] = df_ts2["reach"].cumsum()
                df_all_reach = pd.concat([df_ts1, df_ts2])
                df_all_reach = df_all_reach.reset_index(drop=True)

            else:
                df_ts1, _ = dataframes_ts(
                    df_r1=df_reach1,
                    df_r2=df_reach2,
                    date_max=date_max1,
                    days=days,
                    column_reach=column_reach_sn,
                )
                df_all_reach = df_ts1
                df_all_reach["reach"] = df_all_reach["reach"].cumsum()

        except KeyError as e_1:
            print("".center(60, "="))
            print(e_1)
            print("".center(60, "="))
            print(ERR_SYS + str(exc_info()[0]))
            print("".center(60, "="))
            print(f"Class: {CLASS_NAME} => {METHOD_NAME[4]}")
            print("".center(60, "="))
            traceback.print_exc()

            df_all_reach = pd.DataFrame(columns=["date", "reach", "period", "pos"])

        return df_all_reach



import re
import sys
from copy import deepcopy
from datetime import datetime

import pandas as pd

from wj_analysis.instagram import polarity_distribution

ERR_SYS = "\nSystem error: "


class Hashtags:
    def content_type(self, df_hashtags):
        """
        This method calculates the number of publications of each type
        for a specific hashtag call.

        Parameters
        ----------
        df_hashtags:
            type: DataFrame
            This Pandas DataFrame is obtained by the call of a hashtag using the
            official Instagram API by Facebook
        """

        METHOD_NAME = "content_type"

        HASHTAGS_COLUMNS = ["comments_count", "like_count", "media_type"]

        OUTPUT_COLUMNS = ["media_type", "counts", "avg_engagements_rate", "percentage"]

        try:
            output = pd.DataFrame(columns=OUTPUT_COLUMNS)
            df_contenttype = deepcopy(df_hashtags[HASHTAGS_COLUMNS])
            df_contenttype["engagement_rate"] = (
                df_contenttype["comments_count"] + df_contenttype["like_count"]
            )
            media_type = df_contenttype["media_type"].drop_duplicates()
            total_counts = len(df_contenttype)
            for mt in media_type:
                df_type = df_contenttype[df_contenttype["media_type"] == mt]
                counts = len(df_type)
                avg_engagement_rate = df_type["engagement_rate"].mean()
                percentage = (counts / total_counts) * 100
                output.loc[len(output)] = [mt, counts, avg_engagement_rate, percentage]
            return output
        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=OUTPUT_COLUMNS)

    def sentiment_analysis(self, df_hashtags):
        """
        This method evaluates the sentiment of the text inside each publication that has
        the desire hashtag.

        Parameters
        ----------
        df_hashtags:
            type: DataFrame
            This Pandas DataFrame is obtained by the call of a hashtag using the
            official Instagram API by Facebook
        """

        METHOD_NAME = "sentiment_analysis"

        HASHTAGS_COLUMNS = ["caption"]

        OUTPUT_COLUMNS = ["all", "sentiment"]

        try:
            df_sentimentanalysis = deepcopy(df_hashtags[HASHTAGS_COLUMNS])
            df_sentimentanalysis = df_sentimentanalysis.rename(
                columns={"caption": "text"}
            )
            getpol = polarity_distribution.PolarityDistributionIG(
                df_sentimentanalysis, {}
            )
            getpol.get_polarity()
            return getpol.grouped_polarities(group_by="all")

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=OUTPUT_COLUMNS)

    def time_series(self, df_hashtags, level="hour"):
        """
        This method calculates the frequency that the hashtag had during a period.

        Parameters
        ----------
        df_hashtags:
            type: DataFrame
            This Pandas DataFrame is obtained by the call of a hashtag using the
            official Instagram API by Facebook
        level:
            type: string
            This has to be one of the values of the variable "frequency", will filter
            the way the output would group.
        """

        METHOD_NAME = "time_series"

        HASHTAGS_COLUMNS = ["timestamp", "id"]

        OUTPUT_COLUMNS = ["created_at", "counts", "level"]

        frequency = {
            "business_day": "B",
            "calendar_day": "D",
            "weekly": "W",
            "monthly": "M",
            "15days": "SM",
            "hour": "H",
            "minutes": "T",
            "seconds": "S",
        }

        try:
            df_post_by_date = deepcopy(df_hashtags)
            df_post_by_date = df_post_by_date[["timestamp", "id"]]
            df_post_by_date["timestamp"] = df_post_by_date["timestamp"].apply(
                lambda x: datetime.strptime(x, "%Y-%m-%dT%H:%M:%S+%f")
            )
            for j in range(0, len(df_post_by_date)):
                if level == "calendar_day":
                    date = df_post_by_date["timestamp"][j]
                    df_post_by_date["timestamp"][j] = date.replace(
                        hour=0, minute=0, second=0, microsecond=0
                    )
                elif level == "hour":
                    date = df_post_by_date["timestamp"][j]
                    df_post_by_date["timestamp"][j] = date.replace(
                        minute=0, second=0, microsecond=0
                    )
            df_post_by_date = (
                df_post_by_date.groupby("timestamp")["id"]
                .apply(list)
                .reset_index(name="list_id_post")
            )
            df_post_by_date = df_post_by_date.sort_values("timestamp")

            df_timeseries = deepcopy(df_hashtags[HASHTAGS_COLUMNS])
            df_timeseries["timestamp"] = df_timeseries["timestamp"].apply(
                lambda x: datetime.strptime(x, "%Y-%m-%dT%H:%M:%S+%f")
            )
            output = (
                df_timeseries.groupby(
                    [pd.Grouper(key="timestamp", freq=frequency[level])]
                )
                .size()
                .reset_index(name="counts")
            )
            output = output.rename(columns={"timestamp": "created_at"})

            output = output.sort_values("created_at")
            output["level"] = level

            output["list_post_id"] = ""
            for i in range(0, len(output)):
                for k in range(0, len(df_post_by_date)):
                    if df_post_by_date["timestamp"][k] == output["created_at"][i]:
                        output["list_post_id"][i] = df_post_by_date["list_id_post"][k]
            for i in range(0, len(output)):
                if output["list_post_id"][i] == "":
                    output["list_post_id"][i] = []
            return output

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=OUTPUT_COLUMNS)

    def additional_hashtags(self, df_hashtags, top=10):
        """
        This method filters the hashtags that accompany the original
        and calculates their number of calls.

        Parameters
        ----------
        df_hashtags:
            type: DataFrame
            This Pandas DataFrame is obtained by the call of a hashtag using the
            official Instagram API by Facebook
        top:
            type: int
            Top number of hashtags that want to be return.
        """

        METHOD_NAME = "additional_hashtags"

        HASHTAGS_COLUMNS = ["caption"]

        OUTPUT_COLUMNS = ["hashtags", "counts"]

        try:
            df_aditionalhashtags = deepcopy(df_hashtags[HASHTAGS_COLUMNS])
            df_aditionalhashtags = df_aditionalhashtags.rename(
                columns={"caption": "text"}
            )
            df_aditionalhashtags = df_aditionalhashtags.dropna()
            df_aditionalhashtags["text"] = df_aditionalhashtags["text"].apply(
                lambda text: re.sub(r"[^0-9a-zA-Z#]", " ", text).replace("#", " #")
            )
            hashtags_list = []
            df_aditionalhashtags["text"].apply(
                lambda text: [
                    hashtags_list.append(word) for word in text.split() if "#" in word
                ]
            )
            output = (
                pd.DataFrame(
                    [hashtag.lower() for hashtag in hashtags_list], columns=["hashtags"]
                )["hashtags"]
                .value_counts()
                .reset_index(name="counts")
            )
            output = output[~output["index"].isin(["#"])]
            return output.iloc[1 : top + 1].reset_index(drop=True)

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(columns=OUTPUT_COLUMNS)



import sys
from collections import Counter
from copy import deepcopy

import pandas as pd
from nltk.corpus import stopwords
from sklearn.feature_extraction.text import TfidfVectorizer
from spacy.lang.es.stop_words import STOP_WORDS

from ..common.nlp_utils import CleanText

ERR_SYS = "\nSystem error: "


class WordAnalysis:
    # @profile
    def evaluate(self, df_sentiment, df_posts, threshold=10, group=False, top=10):
        """
        This method evaluates the term frequency and the TF-IDF of a set of post'.

        Parameters
        ----------
        df_sentiment:
            type: DataFrame
            Information of comments by sentiment
        df_posts:
            type: DataFrame
            Information of the post and comments of FB.
            This Pandas DataFrame must have columns 'page_id', 'post_id', 'message',and 'permalink_url'.
        threshold:
            type: int
            Minimum number of comments to be able to evaluate frequency and TF-IDF.
        group:
            type: bool
            Defines if is grouped by "group" or "brand".
        top:
            type: int
            Defines the number of words return in each dictionary.
        """
        METHOD_NAME = "evaluate"

        POSTS_COLUMNS = ["page_id", "post_id", "message", "permalink_url"]

        if group:
            var_analysis = ["group"]
        else:
            var_analysis = ["_object_id", "_object_name"]

        OUTPUT_COLUMNS = [
            "negative_frequency",
            "neutral_frequency",
            "positive_frequency",
            "negative_tfidf",
            "neutral_tfidf",
            "positive_tfidf",
        ]
        OUTPUT_COLUMNS.extend(var_analysis)

        if len(df_sentiment) > 0 and len(df_posts) > 0:
            try:
                self.df_getpolarity = deepcopy(df_posts[POSTS_COLUMNS])
                self.df_getpolarity = pd.merge(
                    self.df_getpolarity, df_sentiment, how="left", on="post_id"
                )
                self.df_getpolarity = self.df_getpolarity.dropna(
                    subset=["sentiment"]
                ).reset_index(drop=True)
                self.df_getpolarity[
                    "negative_count"
                ] = self.df_getpolarity.sentiment.apply(
                    lambda polarity: polarity.get(-1.0, 0.0) + polarity.get(-0.5, 0.0)
                )
                self.df_getpolarity[
                    "neutral_count"
                ] = self.df_getpolarity.sentiment.apply(
                    lambda polarity: polarity.get(0.0, 0.0)
                )
                self.df_getpolarity[
                    "positive_count"
                ] = self.df_getpolarity.sentiment.apply(
                    lambda polarity: polarity.get(1.0, 0.0) + polarity.get(0.5, 0.0)
                )
                if "processed_text" not in self.df_getpolarity.keys():
                    self.df_getpolarity["processed_text"] = self.df_getpolarity[
                        "message"
                    ].apply(
                        lambda msg: CleanText(msg).process_text(
                            mentions=True, hashtags=True, links=True, spec_chars=True
                        )
                    )
                    self.df_getpolarity["processed_text"] = self.df_getpolarity[
                        "processed_text"
                    ].apply(
                        lambda msg: " ".join(
                            [
                                word
                                for word in str(msg).split(" ")
                                if word not in STOP_WORDS
                            ]
                        )
                    )
                self.df_getpolarity = self.df_getpolarity.dropna(
                    subset=["processed_text"]
                )
                self.df_getpolarity = self.df_getpolarity.drop(
                    self.df_getpolarity[
                        self.df_getpolarity["processed_text"] == ""
                    ].index
                )
                self.df_output = pd.DataFrame(columns=OUTPUT_COLUMNS)
                item_list = self.df_getpolarity[var_analysis[0]].drop_duplicates()
                for item in item_list:
                    df_tfidf = self.df_getpolarity[
                        self.df_getpolarity[var_analysis[0]] == item
                    ]
                    temp = [
                        self.frequency(
                            df_tfidf[df_tfidf["negative_count"] >= threshold], top
                        ),
                        self.frequency(
                            df_tfidf[df_tfidf["neutral_count"] >= threshold], top
                        ),
                        self.frequency(
                            df_tfidf[df_tfidf["positive_count"] >= threshold], top
                        ),
                        self.tfidf(
                            df_tfidf[df_tfidf["negative_count"] >= threshold], top
                        ),
                        self.tfidf(
                            df_tfidf[df_tfidf["neutral_count"] >= threshold], top
                        ),
                        self.tfidf(
                            df_tfidf[df_tfidf["positive_count"] >= threshold], top
                        ),
                        item,
                    ]
                    if not group:
                        temp.extend([df_tfidf[var_analysis[1]].iloc[0]])
                    self.df_output.loc[len(self.df_output)] = temp
            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                self.df_output = pd.DataFrame(columns=OUTPUT_COLUMNS)

    def tfidf(self, df, top):
        """
        This method evaluates TF-IDF on the DataFrame'.

        Parameters
        ----------
        df:
            type: DataFrame

        top:
            type: int
            Defines the number of words return in each dictionary.

        Returns
            df: DataFrame
            type: DataFrame
            DataFrame with the TF-IDF of each word.
        -------
        """
        if len(df) < 2:
            return {}
        else:
            vectorizer = TfidfVectorizer()
            vectors = vectorizer.fit_transform(df["processed_text"])
            feature_names = vectorizer.get_feature_names()
            dense = vectors.todense()
            denselist = dense.tolist()
            return (
                pd.DataFrame(denselist, columns=feature_names)
                .sum(axis=0, skipna=True)
                .sort_values(ascending=False)
                .head(top)
                .to_dict()
            )

    def frequency(self, df, top):
        """
        This method evaluates the term frequency and the TF-IDF of a set of post'.

        Parameters
        ----------
        df:
            type: DataFrame

        top:
            type: int
            Defines the number of words return in each dictionary.

        Returns
            dict: Dictionary
            type: Dictionary
            Dictionary with the top frequencies of each word.
        """
        if df.empty:
            return {}
        else:
            text = " ".join(list(df["processed_text"]))
            bagofwords = text.split(" ")
            tf_dict = {}
            bagofwordscount = len(bagofwords)
            for word, count in Counter(bagofwords).items():
                tf_dict[word] = count / float(bagofwordscount)
            return dict(sorted(tf_dict.items(), key=lambda x: x[1], reverse=True)[:top])



import sys
from collections import Counter
from copy import deepcopy
from datetime import timedelta

import pandas as pd
from numpy import unique

from ..common import general_utils
from ..common.nlp_utils import STTM, CleanText, Features, Polarity

ERR_SYS = "\nSystem error: "


class STTMgroupsIG:
    """
    This class groups texts into groups of similar topics.
    """

    def __init__(self, df_comments, account_ids=[], account_names=[]):
        """
        This method computes the DataFrame 'df_comments_full'
        which contains all the information of the instagram comments.

        Parameters
        ----------
        df_comments:
            type: DataFrame
            Information of the instagram comments.
            This Pandas DataFrame must have columns 'post_owner_id', 'post_owner_username', 'text'.
        account_ids=[]:
            type: list
            list of accounts ids in case filtering by account id is needed.
        account_names=[]:
            type: list
            list of accounts names in case filtering by account name is needed.
        """

        METHOD_NAME = "__init__"
        df_comments_full = deepcopy(df_comments)
        self.df_comments_full = df_comments_full

        try:

            if "post_owner_username" not in self.df_comments_full.keys():
                print(
                    "Warning: Class: {self.__str__()}\nMethod: {METHOD_NAME}' no 'post_owner_username' column in DataFrame"
                )
                self.df_comments_full["post_owner_username"] = "no name"
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            if "post_owner_username" not in self.df_comments_full.keys():
                self.df_comments_full["post_owner_username"] = "no name"

        if account_ids:
            self.df_comments_full = self.df_comments_full[
                self.df_comments_full.post_owner_id.isin(account_ids)
            ]
        elif account_names:
            self.df_comments_full = self.df_comments_full[
                self.df_comments_full.post_owner_username.isin(account_ids)
            ]

        try:
            if df_comments.empty:
                print("Warning: input data DataFrame is empty.")
        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_comments_full = pd.DataFrame(columns=[""])

    def remove_from_text(self, text, toremove):
        """
        removes words from text.

        Parameters
        ----------
        text
            type: list of strings
            text of the post or comment.

        toremove:
            type: list of strings
            list of words to remove from text.

        Returns
        -------
        list of strings
        """
        text_copy = deepcopy(text)
        for word in text:
            if word in toremove:
                text_copy.remove(word)
        return text_copy

    def get_sttm_groups(self, list_remove=[], min_texts=100):
        """
        Uses Short Text Topic Modelling to group the texts.
        Returns the input DataFrame with an additional column (sttm_group) with the group number to which each text was assigned.

        Parameters
        ----------
        with_remove (optional, default False)
            type: boolean
            set to True if user wants to remove a list of words from the texts before grouping.

        list_remove
            type: list
            list of words that the user wants to remove

        min_texts
            type: int
            minimum number of texts in the DataFrame (after cleaning and dropping empty texts) that is required to proceed with the sttm analysis.

        Returns
        -------
        DataFrame with an additional column (sttm_groups)
        """

        METHOD_NAME = "get_sttm_groups"

        try:

            # cleaning text:
            if "processed_text" not in self.df_comments_full.keys():
                self.df_comments_full["processed_text"] = self.df_comments_full[
                    "text"
                ].apply(
                    lambda msg: CleanText(msg).process_text(
                        mentions=True, hashtags=True, links=True, spec_chars=True
                    )
                )

            # drop empty comments
            self.df_comments_full = self.df_comments_full.dropna(
                subset=["processed_text"]
            )
            self.df_comments_full = self.df_comments_full.drop(
                self.df_comments_full[
                    self.df_comments_full["processed_text"] == ""
                ].index
            )

            if list_remove:
                self.df_comments_full["processed_text"] = self.df_comments_full[
                    "processed_text"
                ].apply(lambda x: str(x).split(" "))
                self.df_comments_full["processed_text"] = self.df_comments_full[
                    "processed_text"
                ].apply(lambda x: self.remove_from_text(x, list_remove))
                self.df_comments_full["processed_text"] = self.df_comments_full[
                    "processed_text"
                ].apply(lambda msg: str(msg))

            # drop empty comments
            self.df_comments_full = self.df_comments_full.dropna(
                subset=["processed_text"]
            )
            self.df_comments_full = self.df_comments_full.drop(
                self.df_comments_full[
                    self.df_comments_full["processed_text"] == ""
                ].index
            )
            self.df_comments_full[
                "len_text"
            ] = self.df_comments_full.processed_text.apply(lambda msg: len(msg))
            self.df_comments_full = self.df_comments_full[
                self.df_comments_full["len_text"] != 0
            ]

            # getting the polarity of the clean text
            if "polarity" not in self.df_comments_full.keys():
                self.df_comments_full = Polarity().polarity(self.df_comments_full)

            # tokenized text:
            if "tokenized_text" not in self.df_comments_full.keys():
                getfeatures = Features()
                self.df_comments_full[
                    "tokenized_text"
                ] = self.df_comments_full.processed_text.apply(
                    lambda msg: getfeatures.pos_tags(msg)["words"]
                )

            # drop empty comments
            self.df_comments_full = self.df_comments_full.dropna(
                subset=["tokenized_text"]
            )
            self.df_comments_full = self.df_comments_full.drop(
                self.df_comments_full[
                    self.df_comments_full["tokenized_text"] == ""
                ].index
            )
            self.df_comments_full[
                "len_text"
            ] = self.df_comments_full.tokenized_text.apply(lambda msg: len(msg))
            self.df_comments_full = self.df_comments_full[
                self.df_comments_full["len_text"] != 0
            ]

            # getting sttm groups
            if self.df_comments_full.tokenized_text.shape[0] > min_texts:
                self.df_comments_full = STTM(self.df_comments_full).sttm_model()
            else:
                print(
                    "warning: in",
                    f"Class: {self.__str__()}\nMethod: {METHOD_NAME}",
                    "not enough texts in DataFrame for STTM grouping. Returning value 1 for group column for sttm_group",
                )
                self.df_comments_full["sttm_group"] = 1

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            if "polarity" not in self.df_comments_full.keys():
                self.df_comments_full["polarity"] = ""
            if "sttm_group" not in self.df_comments_full.keys():
                self.df_comments_full["sttm_group"] = ""

    def get_sttm_groups_withremove(
        self, df_preprocessed_sttm, list_toremove=[], min_texts=100
    ):
        """
        Uses Short Text Topic Modelling to group the texts.
        Returns the input DataFrame with an additional column (sttm_group) with the group number to which each text was assigned.

        Parameters
        ----------
        df_preprocessed_sttm
            type: DataFrame
            Preprocessed DataFrame so that the grouping can be redone without redoing the tokenizing of the text..

        list_toremove
            type: list
            list of words that the user wants to remove.

        min_texts
            type: int
            minimum number of texts in the DataFrame (after cleaning and dropping empty texts) that is required to proceed with the sttm analysis.

        Returns
        -------
        DataFrame with column (sttm_groups)
        """

        METHOD_NAME = "get_sttm_groups_withremove"
        self.df_preprocessed_sttm = deepcopy(df_preprocessed_sttm)

        try:
            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: x[2:-2])
            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: str(x).split("', '"))

            self.df_preprocessed_sttm["tokenized_text"] = self.df_preprocessed_sttm[
                "tokenized_text"
            ].apply(lambda x: self.remove_from_text(x, list_toremove))

            # drop empty comments
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.dropna(
                subset=["tokenized_text"]
            )
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.drop(
                self.df_preprocessed_sttm[
                    self.df_preprocessed_sttm["tokenized_text"] == ""
                ].index
            )
            self.df_preprocessed_sttm = self.df_preprocessed_sttm.drop(
                self.df_preprocessed_sttm[
                    self.df_preprocessed_sttm["tokenized_text"] == " "
                ].index
            )
            self.df_preprocessed_sttm[
                "len_text"
            ] = self.df_preprocessed_sttm.tokenized_text.apply(lambda msg: len(msg))
            self.df_preprocessed_sttm = self.df_preprocessed_sttm[
                self.df_preprocessed_sttm["len_text"] != 0
            ]

            # getting sttm groups
            if self.df_preprocessed_sttm.tokenized_text.shape[0] > min_texts:
                self.df_preprocessed_sttm = STTM(self.df_preprocessed_sttm).sttm_model()
            else:
                print(
                    "warning: in",
                    f"Class: {self.__str__()}\nMethod: {METHOD_NAME}",
                    "\n Not enough texts in DataFrame for STTM grouping. Returning value 1 for group column for sttm_group",
                )
                self.df_preprocessed_sttm["sttm_group"] = 1

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")
            self.df_preprocessed_sttm["sttm_group"] = ""

    def counter_of_words(self, words, mention, n_words=10):
        """
        Counts the elements in the list 'words' and returns
        the 'n_words' most frequent.

        Parameters
        ----------
        words
            type: list
            List of words to count.
        mention:
            type: str
            Mention to delete from the counter to avoid overcounting
            mentions in responses to a certain account.
        n_words:
            type: int
            Number of most frequent words to return

        Returns
        -------
        dict
        """

        counter = Counter(words)
        try:
            del counter["@" + mention.lower()]
        except Exception:
            pass
        return dict(counter.most_common(n_words))

    def get_percentage(self, counter):
        """
        Computes the frequency (percentage) of the elements in the
        dictionary 'counter'.

        Parameters
        ----------
        counter
            type: dict
            Dictionary of elements to count.

        Returns
        -------
        dict
        """

        percentage = Counter(counter)
        percentage = dict(
            [
                (i, perc / sum(percentage.values()) * 100.0)
                for i, perc in percentage.most_common()
            ]
        )
        return percentage

    def message_to_group_columns(self):
        """
        This function constructs the columns to group the dataframe
        'data' for audiencies visualizations.

        Parameters
        ----------
        data:
            type: DataFrame
            DataFrame containing information about texts

        Returns
        -------
        DataFrame
        """

        data_texts = deepcopy(self.df_comments_full)

        if "__date_cot" not in data_texts.keys():
            data_texts["__date_cot"] = pd.to_datetime(
                data_texts["created_at_utc"]
            ) - timedelta(hours=5)

        if "post_owner_username" in data_texts.keys():
            data_texts = data_texts[
                [
                    "sttm_group",
                    "tokenized_text",
                    "post_owner_username",
                    "__date_cot",
                    "polarity",
                ]
            ]
            data_texts = data_texts.rename(columns={"post_owner_username": "__mention"})
        else:
            data_texts = data_texts[
                ["sttm_group", "tokenized_text", "__date_cot", "polarity"]
            ]
            data_texts["__mention"] = ""

        self.data_texts = data_texts

    def remove_insignificant_groups(
        self,
        df_sttm_groups,
        min_num_docs=15,
        min_group_percent=10,
        max_num_groups=5,
        min_num_groups=2,
    ):

        """
        this function removes the groups that have very little relevance due to the small number of elements (comments) or the small percentage that they represent.

        Parameters
        ----------
        df_sttm_groups:
            type: DataFrame
            DataFrame containing the comments and the group they belong to.

        min_num_docs:
            type:int
            minimum number of comments that a group has to have in order to be show in the platform.

        min_group_percent:
            type: float
            percentage threshold that a group has to represent in order to be show in the platform.

        max_num_groups:
            type:int
            maximun number of groups shown by the platform.

        min_num_groups:
            type:int
            mminimum number of groups shown by the platform.

        Returns
        -------
        DataFrame with only the groups that comply with the criteria.
        """
        METHOD_NAME = "remove_insignificant_groups"
        if min_num_groups > max_num_groups:
            min_num_groups = max_num_groups

        if df_sttm_groups.shape[0] < min_num_groups:
            min_num_groups = df_sttm_groups.shape[0]
            max_num_groups = df_sttm_groups.shape[0]

        try:
            if (
                df_sttm_groups["__n_docs"][min_num_groups - 1] <= min_num_docs
                or df_sttm_groups["__percentage"][min_num_groups - 1]
                <= min_group_percent
            ):
                df_sttm_groups = df_sttm_groups.head(min_num_groups)

            else:
                df_sttm_groups = df_sttm_groups[
                    df_sttm_groups["__n_docs"] >= min_num_docs
                ]

                df_sttm_groups = df_sttm_groups[
                    df_sttm_groups["__percentage"] >= min_group_percent
                ]

                df_sttm_groups = df_sttm_groups.head(max_num_groups)

        except Exception as e:
            print(e)
            error_1 = sys.exc_info()[0]
            print(ERR_SYS + str(error_1))
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}")

        return df_sttm_groups

    def get_examples_from_sttm_groups(
        self,
        df_data_grouped,
        group,
        num_examples=3,
        num_top_words_group=5,
        min_text_lenght=1,
    ):
        """
        this function returns examples of comments in each sttm group.

        Parameters
        ----------

        df_data_grouped:
            type: pandas DataFrame
            max number ofDataFrame with grouped information for each sttm group.

        group:
            type:int
            group number.

        num_examples: (optional, default=3)
            type:int
            number of examples selected for each sttm group.

        num_top_words_group: (optional, default=5)
            type:int
            number of most frequent words for each group that will be used as a selection crtiteria to decide wether or not a comment will be used as an example comment for the group.

        min_text_lenght: (optional, default=1)
            type:int
            minimum number of tokens in the selected comment examples.

        Returns
        -------
        a list of IDs of the selected example comments
        """

        df_comments = deepcopy(self.df_comments_full)
        post_examples_id = []

        group_word_list = [
            *df_data_grouped[df_data_grouped["sttm_group"] == group]["__words"].iloc[0]
        ]
        num_top_words_group = min(num_top_words_group, len(group_word_list))

        num_examples = min(
            num_examples, df_comments[df_comments["sttm_group"] == group].shape[0]
        )
        group_word_list = group_word_list[:num_top_words_group]

        df_comments["len_comments"] = df_comments.tokenized_text.apply(lambda x: len(x))
        df_comments["word_in_comment"] = df_comments.tokenized_text.apply(
            lambda x: True if (set(x) & set(group_word_list)) else False
        )
        df_sample = df_comments.where(
            (df_comments["len_comments"] >= min_text_lenght)
            & (df_comments["word_in_comment"] == True)
        )
        df_sample = df_sample.dropna(subset=["len_comments", "word_in_comment"])
        if df_sample.shape[0] > num_examples:
            post_examples_id = df_sample.comment_id.sample(num_examples).values.tolist()
        else:
            post_examples_id = df_sample.comment_id.values.tolist()
        return post_examples_id

    def get_grouped_dataframe(self):
        """
        Groups the DataFrame 'data' into a DataFrame containing information
        abput the groups of texts.

        Parameters
        ----------
        data
            type: DataFrame
            DataFrame containing information about the texts.

        Returns
        -------
        DataFrame
        """
        data_grouped = deepcopy(
            self.data_texts[
                ["sttm_group", "tokenized_text", "__mention", "__date_cot", "polarity"]
            ]
        )

        data_grouped["polarity"] = data_grouped["polarity"].apply(lambda pol: str(pol))

        data_grouped["__weekday"] = data_grouped["__date_cot"].apply(
            lambda d: d.strftime("%A")
        )

        data_grouped["__n_docs"] = 1

        data_grouped = (
            data_grouped[
                [
                    "sttm_group",
                    "__n_docs",
                    "tokenized_text",
                    "__mention",
                    "__weekday",
                    "polarity",
                ]
            ]
            .groupby("sttm_group")
            .agg(
                {
                    "__n_docs": "count",
                    "tokenized_text": "sum",
                    "__mention": "last",
                    "__weekday": lambda x: ",".join(x)
                    if len(unique(data_grouped["__weekday"])) > 1
                    else "",
                    "polarity": lambda x: ",".join(x),
                }
            )
        )

        data_grouped["__percentage"] = (
            data_grouped["__n_docs"] * 100 / sum(data_grouped["__n_docs"])
        )

        data_grouped = data_grouped.rename(columns={"tokenized_text": "__words"})

        data_grouped["__words"] = data_grouped.apply(
            lambda row: self.counter_of_words(row["__words"], row["__mention"]), axis=1
        )

        data_grouped = data_grouped.reset_index().sort_values(
            "__percentage", ascending=False
        )
        data_grouped = self.remove_insignificant_groups(data_grouped)

        data_grouped = data_grouped.drop(columns=["__mention"])
        data_grouped["__weekday"] = data_grouped["__weekday"].apply(
            lambda wd: wd.split(",")
        )
        data_grouped = data_grouped.rename(columns={"__weekday": "__presence"})
        data_grouped["__presence"] = data_grouped["__presence"].apply(
            lambda p: self.get_percentage(p)
        )

        data_grouped["polarity"] = data_grouped["polarity"].apply(
            lambda wd: wd.split(",")
        )
        data_grouped = data_grouped.rename(
            columns={"polarity": "__polarity_distribution"}
        )
        data_grouped["__polarity_distribution"] = data_grouped[
            "__polarity_distribution"
        ].apply(lambda p: self.get_percentage(p))

        data_grouped["__examples_comments"] = data_grouped.sttm_group.apply(
            lambda g: self.get_examples_from_sttm_groups(
                data_grouped, g, num_examples=3
            )
        )

        # reorganize and rename sttm groups
        data_grouped = data_grouped.reset_index()
        data_grouped["sttm_group"] = data_grouped.index
        data_grouped["sttm_group"] = data_grouped["sttm_group"].apply(lambda i: i + 1)
        data_grouped = data_grouped.drop(columns="index")
        self.data_grouped = data_grouped

        return data_grouped



import sys
from copy import deepcopy
from datetime import timedelta

import pandas as pd

from ..common import general_utils, metric_transformation

ERR_SYS = "\nSystem error: "


class EngagementRateIG:
    def __init__(self, df_posts, df_pages, groups, mode="status"):
        """
        This method computes the DataFrame 'df_posts_full' with the columns
        'group' and 'followers' which is a count of the fans in the day the post was made.

        Parameters
        ----------
        df_posts:
            type: DataFrame
            this Pandas DataFrame muts have columns
            'owner_id' and 'date_utc'.
        df_pages:
            type: DataFrame
            this Pandas DataFrame muts have columns
            'followers', 'userid' and 'date'.
        groups:
            type: dict
            Maps the groups (client, compentition, archetype, trends) to the
            corresponding page ids for each group.
        """

        METHOD_NAME = "__init__"

        METHOD_NAME = "__init__"

        self.mode = mode

        if self.mode == "status":
            PAGES_COLUMNS = ["followers", "userid", "date"]

            POSTS_COLUMNS = [
                "owner_id",
                "owner_username",
                "date_utc",
                "shortcode",
                "likes_count",
                "comment_count",
                "caption_hashtags",
                "typename",
                "caption_mentions",
                "caption",
            ]
            OUTPUT_COLUMNS = [
                "followers",
                "owner_id",
                "owner_username",
                "group",
                "created_at",
                "shortcode",
                "likes_count",
                "comment_count",
                "caption_hashtags",
                "typename",
                "caption_mentions",
                "caption",
            ]

            if len(df_posts) > 0 and len(df_pages) > 0:
                try:
                    df_fan_count = deepcopy(df_pages[PAGES_COLUMNS])
                    df_fan_count["userid"] = df_fan_count["userid"].apply(
                        lambda uid: str(uid)
                    )

                    df_fan_count["date"] = pd.to_datetime(
                        df_fan_count["date"], format="%Y-%m-%dT%H:%M:%S"
                    )
                    df_fan_count["date"] = pd.to_datetime(
                        df_fan_count["date"]
                    ) - timedelta(hours=5)
                    df_fan_count["date"] = df_fan_count["date"].apply(lambda d: d.date)
                    df_fan_count = (
                        df_fan_count.groupby(["userid", "date"]).last().reset_index()
                    )

                    df_fan_count = df_fan_count.rename(columns={"userid": "owner_id"})

                    df_posts_full = deepcopy(df_posts[POSTS_COLUMNS])
                    df_posts_full["owner_id"] = df_posts_full["owner_id"].apply(
                        lambda uid: str(uid)
                    )

                    df_posts_full["date"] = pd.to_datetime(
                        df_posts_full["date_utc"], format="%Y-%m-%dT%H:%M:%S"
                    )
                    df_posts_full["created_at"] = pd.to_datetime(
                        df_posts_full["date"]
                    ) - timedelta(hours=5)
                    df_posts_full["date"] = pd.to_datetime(
                        df_posts_full["date"]
                    ) - timedelta(hours=5)
                    df_posts_full["date"] = df_posts_full["date"].apply(
                        lambda d: d.date
                    )
                    df_posts_full = pd.merge(
                        df_posts_full, df_fan_count, on=["owner_id", "date"], how="left"
                    )
                    df_posts_full["followers"] = df_posts_full.sort_values(
                        ["owner_id", "date"]
                    )["followers"].fillna(method="bfill")

                    df_posts_full = df_posts_full.sort_values("date").drop_duplicates(
                        subset=["shortcode"], keep="last"
                    )

                    df_posts_full["group"] = df_posts_full["owner_id"].apply(
                        lambda pid: general_utils.get_group(pid, groups)
                    )

                    self.df_posts_full = df_posts_full

                except Exception as e:
                    exception_type = sys.exc_info()[0]
                    print(ERR_SYS + str(exception_type))
                    print(e)
                    print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
                    self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS)

            else:
                print("Warning: One of the DataFrames is empty. It cannot be computed.")
                self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS)
        elif self.mode == "hashtags":
            HASHTAG_COLUMNS = [
                "caption",
                "comments_count",
                "like_count",
                "media_type",
                "timestamp",
            ]
            OUTPUT_COLUMNS = [
                "followers",
                "group",
                "created_at",
                "likes_count",
                "comment_count",
                "typename",
                "caption",
            ]
            try:
                df_hashtag_full = deepcopy(
                    df_posts[HASHTAG_COLUMNS]
                )  # df_hashtag camuflado
                df_hashtag_full = df_hashtag_full.rename(
                    columns={
                        "comments_count": "comment_count",
                        "like_count": "likes_count",
                        "media_type": "typename",
                        "timestamp": "date_utc",
                    }
                )

                df_hashtag_full = df_hashtag_full.drop_duplicates()
                df_hashtag_full["followers"] = 1
                df_hashtag_full["group"] = "hashtags"
                df_hashtag_full["date"] = pd.to_datetime(
                    df_hashtag_full["date_utc"], format="%Y-%m-%dT%H:%M:%S"
                )
                df_hashtag_full["created_at"] = pd.to_datetime(
                    df_hashtag_full["date"]
                ) - timedelta(hours=5)
                self.df_posts_full = df_hashtag_full[OUTPUT_COLUMNS]
            except Exception as e:
                exception_type = sys.exc_info()[0]
                print(ERR_SYS + str(exception_type))
                print(e)
                print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
        else:
            self.df_posts_full = pd.DataFrame(columns=OUTPUT_COLUMNS)

    def by_reach(self):
        """
        This method computes the engagement rate for every post based on the
        reach of the post. It stores it on the column
        'engagement_rate_by_reach' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_reach"

        df_posts_full = self.df_posts_full
        try:
            if self.mode == "hashtags":
                df_posts_full["engagement_rate_by_post"] = df_posts_full.apply(
                    lambda row: (row.likes_count + row.comment_count) / row.followers,
                    axis=1,
                )

            else:
                df_posts_full["engagement_rate_by_post"] = df_posts_full.apply(
                    lambda row: 100
                    * (row.likes_count + row.comment_count)
                    / row.followers,
                    axis=1,
                )
                METRIC = "engagement_rate_by_post"
                ITEM_COLUMN = "owner_id"
                df_posts_full = df_posts_full.sort_values(by=["created_at"])
                df_posts_full = metric_transformation.MetricCategorization(
                    df_posts_full, METRIC, ITEM_COLUMN
                ).categorize()

            return df_posts_full.dropna(subset=["engagement_rate_by_post"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns)
                + ["engagement_rate_by_post", "boundary", "rel_engagement_rate_by_post"]
            )

    def by_post(self):
        """
        This method computes the engagement rate for every post based on the
        number of followers of the account. It stores it on the column
        'engagement_rate_by_post' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_reach"

        df_posts_full = self.df_posts_full
        try:
            df_posts_full["engagement_rate_by_post"] = df_posts_full.apply(
                lambda row: 100 * (row.likes_count + row.comment_count) / row.followers,
                axis=1,
            )

            METRIC = "engagement_rate_by_post"
            ITEM_COLUMN = "owner_id"
            df_posts_full = df_posts_full.sort_values(by=["created_at"])
            df_posts_full = metric_transformation.MetricCategorization(
                df_posts_full, METRIC, ITEM_COLUMN
            ).categorize()

            return df_posts_full.dropna(subset=["engagement_rate_by_post"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns)
                + ["engagement_rate_by_post", "boundary", "rel_engagement_rate_by_post"]
            )

    def by_impressions(self):
        """
        This method computes the engagement rate for every post based on the
        impressions of the post. It stores it on the column
        'engagement_rate_by_impressions' on the input Pandas DataFrame 'df_posts_full'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_impressions"

        df_posts_full = self.df_posts_full
        try:
            df_posts_full["engagement_rate_by_impressions"] = df_posts_full.apply(
                lambda row: 100
                * (row.likes_count + row.comment_count)
                / row.impressions_count,
                axis=1,
            )
            return df_posts_full.dropna(subset=["engagement_rate_by_impressions"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns) + ["engagement_rate_by_impressions"]
            )

    def by_day(self, grouped=True):
        """
        This method computes the engagement rate by day for each account
        based on the number of followers. It stores it on the column
        'engagement_rate_by_impressions' on the output Pandas DataFrame 'df_eng_by_day'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account by day.
            If True returns the DataFrame by group by day, default=True

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_day"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_day = deepcopy(df_posts_full)

            df_post_by_date = df_eng_by_day
            df_post_by_date = df_post_by_date[["date", "mediaid"]]
            df_post_by_date = (
                df_post_by_date.groupby("date")["mediaid"]
                .apply(list)
                .reset_index(name="list_id_post")
            )
            df_post_by_date = df_post_by_date.list_id_post

            df_eng_by_day = (
                df_eng_by_day[
                    [
                        "owner_id",
                        "owner_username",
                        "group",
                        "date",
                        "comment_count",
                        "likes_count",
                        "followers",
                    ]
                ]
                .groupby(["owner_id", "owner_username", "date"])
                .agg(
                    {
                        "group": "last",
                        "comment_count": "sum",
                        "likes_count": "sum",
                        "followers": "max",
                    }
                )
            )
            df_eng_by_day["engagement_rate_by_day"] = df_eng_by_day.apply(
                lambda row: 100 * (row.likes_count + row.comment_count) / row.followers,
                axis=1,
            )
            df_eng_by_day = df_eng_by_day.reset_index()

            if grouped:
                df_eng_by_day = (
                    df_eng_by_day[["group", "date", "engagement_rate_by_day"]]
                    .groupby(["group", "date"])
                    .mean()
                )
                df_eng_by_day = df_eng_by_day.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_day = df_eng_by_day.rename(
                    columns={"owner_id": "_object_id", "owner_username": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_day"
            df_eng_by_day = df_eng_by_day.sort_values(by=["date"])
            df_eng_by_day = metric_transformation.MetricCategorization(
                df_eng_by_day, METRIC, ITEM_COLUMN
            ).categorize()
            df_eng_by_day["list_post_id"] = df_post_by_date

            return df_eng_by_day[
                PAGE_COLUMNS
                + [
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                    "list_post_id",
                ]
            ].dropna(subset=["engagement_rate_by_day"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "date",
                    "engagement_rate_by_day",
                    "boundary",
                    "rel_engagement_rate_by_day",
                ]
            )

    def by_views(self):
        """
        This method computes the engagement rate for every video post based on the
        views. It stores it on the column 'engagement_rate_by_post'
        on the output Pandas DataFrame 'df_eng_by_views'.

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_views"

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_views = deepcopy(df_posts_full)
            df_eng_by_views = df_eng_by_views[df_eng_by_views.is_video == True]
            if len(df_eng_by_views) > 0:
                df_eng_by_views["engagement_rate_by_views"] = df_eng_by_views.apply(
                    lambda row: 100
                    * (row.likes_count + row.comment_count)
                    / row.video_view_count,
                    axis=1,
                )
                return df_eng_by_views.dropna(subset=["engagement_rate_by_views"])
            else:
                print("There are no posts with videos.")

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=list(df_posts_full.columns) + ["engagement_rate_by_views"]
            )

    def by_accounts(self, grouped=True):
        """
        This method computes the engagement rate by page
        based on the number of followers. It stores it on the column
        'engagement_rate_by_accounts' on the output Pandas DataFrame 'df_eng_by_accounts'.

        Parameters
        ----------
        grouped:
            type: bool
            If False retunrs the DataFrame by account.
            If True returns the DataFrame by group, default=True

        Returns
        -------
        DataFrame
        """

        METHOD_NAME = "by_accounts"

        PAGE_COLUMNS = []
        if not grouped:
            PAGE_COLUMNS = ["owner_id", "owner_username"]

        df_posts_full = self.df_posts_full
        try:
            df_eng_by_accounts = deepcopy(df_posts_full)

            df_eng_by_accounts = (
                df_eng_by_accounts[
                    [
                        "owner_id",
                        "owner_username",
                        "group",
                        "comment_count",
                        "likes_count",
                        "followers",
                    ]
                ]
                .groupby(["owner_id", "owner_username"])
                .agg(
                    {
                        "group": "last",
                        "comment_count": "sum",
                        "likes_count": "sum",
                        "followers": "max",
                    }
                )
            )
            df_eng_by_accounts[
                "engagement_rate_by_accounts"
            ] = df_eng_by_accounts.apply(
                lambda row: 100 * (row.likes_count + row.comment_count) / row.followers,
                axis=1,
            )
            df_eng_by_accounts = df_eng_by_accounts.reset_index()

            if grouped:
                df_eng_by_accounts = (
                    df_eng_by_accounts[["group", "engagement_rate_by_accounts"]]
                    .groupby(["group"])
                    .mean()
                )
                df_eng_by_accounts = df_eng_by_accounts.reset_index()
                PAGE_COLUMNS = []
                ITEM_COLUMN = "group"
            else:
                df_eng_by_accounts = df_eng_by_accounts.rename(
                    columns={"owner_id": "_object_id", "owner_username": "_object_name"}
                )
                PAGE_COLUMNS = ["_object_id", "_object_name"]
                ITEM_COLUMN = "_object_id"

            METRIC = "engagement_rate_by_accounts"
            df_eng_by_accounts = df_eng_by_accounts.sort_values(by=["group"])
            df_eng_by_accounts = metric_transformation.MetricCategorization(
                df_eng_by_accounts, METRIC, ITEM_COLUMN
            ).categorize()

            return df_eng_by_accounts[
                PAGE_COLUMNS
                + [
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ]
            ].dropna(subset=["engagement_rate_by_accounts"])

        except Exception as e:
            exception_type = sys.exc_info()[0]
            print(ERR_SYS + str(exception_type))
            print(e)
            print(f"Class: {self.__str__()}\nMethod: {METHOD_NAME}\n")
            return pd.DataFrame(
                columns=[
                    "_object_id",
                    "_object_name",
                    "group",
                    "engagement_rate_by_accounts",
                    "boundary",
                    "rel_engagement_rate_by_accounts",
                ],
                dtype=object,
            )
